#!/usr/bin/python
#-*-coding:utf-8-*-
"""
    client xmlrpc renvoyant les description des onglets de l'utilisateur posh courant
    appel avec :
        python client.py <infos utilisateur au format Json>

"""
import commands
import json
import xmlrpclib
import sys
from posh.config.config import XMLRPC_IP_SERVER, XMLRPC_PORT, LOCAL_IP
from posh.thumbs.initthumbs import InitThumbs
from posh.thumbs.exportthumbs import ExportThumbs
from posh.logger.logger import log

isdebug = True

def debug(msg, loglevel=0):
    if type(msg) != str:
        msg = str(msg)
    if isdebug:
        log("Client.py : %s" % msg, loglevel)

def encode_value(result):
    try:
        return result.encode('utf-8')
    except:
        return result

def return_result(result):
    """
        renvoie le résultat sous forme de print pour passage au php
    """
    result = encode_value(result).strip()
    debug("+ On renvoie un résultat au php : ")
    debug(result)
    print result

class EnvoleProfilClient:
    """
        client pour les appels profil utilisateur
        desktop: bureau utilisateur
        thumbs : les onglets de l'utilisateur
        export : export d'onglet
    """

    def __init__(self, arguments):
        try:
            self.opt_parse(arguments)
            self.make_server()
        except Exception, msg:
            debug("- ERREUR : dans l'init de EnvoleProfileClient")
            debug(msg.message)

    def user_datas_opt_parse(self, arguments):
        """
            parse des options contenant des attributs utilisateurs
        """
        self.user_datas = self.decode_user_datas(arguments[2])
        self.username = self.user_datas['user']
        if self.user_datas.has_key('details'):
            self.user_datas = self.user_datas['details']
        if self.user_datas.has_key('infos'):
            self.user_datas = self.user_datas['infos']
        debug("Username : %s" % self.username)
        debug("User_datas : %s" % self.user_datas, 1)

    def thumb_exporter_opt_parse(self, arguments):
        """
            parser pour l'export d'onglet
        """
        self.username, self.user_id, self.session_pt, self.thumb_id, self.thumb_name, self.thumb_indice = arguments[2:]
        debug("User_id : %s" % self.user_id)
        debug("Thumb_id : %s" % self.thumb_id, 1)
        debug("Thum_name : %s" %  self.thumb_name, 1)
        debug("Thumb_indice : %s" % self.thumb_indice, 1)
        debug("Session_PT : %s" % self.session_pt, 1)

    def opt_parse(self, arguments):
        """
            parse les arguments passés à l'éxécution du fichier
            récupère le mode et parse selon le mode
        """
        self.mode = arguments[1]
        debug("* On appelle l'action %s" % self.mode.upper())
        if self.mode == 'desktop' : self.user_datas_opt_parse(arguments)
        elif self.mode == "thumbs" : self.user_datas_opt_parse(arguments)
        elif self.mode == "exporter" : self.thumb_exporter_opt_parse(arguments)
        else: self.user_datas_opt_parse(arguments)

    def make_server(self, ip=XMLRPC_IP_SERVER, port=XMLRPC_PORT):
        """
            instancie un client xmlrpc pour appeler le UserContext
        """
        debug("* On construit le serveur xmlrpclib")
        self.server = xmlrpclib.Server('http://%s:%s/' % (ip,port))

    def decode_user_datas(self, user_datas_str):
        """
            décode les données utilisateurs envoyées au format Json
        """
        try:
            return json.read(user_datas_str)
        except Exception, error:
            debug("- ERREUR : json n'a pas pu décoder les informations utilisateurs")
            debug(error.message)
            return {}

    def get_desktop_items(self):
        """
            appel les infos pour la mise en forme des boutons du bureau utilisateur
        """
        try:
            result = self.server.get_desktop_items(self.username, self.user_datas)
        except Exception, error:
            debug("- ERREUR : au contact du serveur de profil  à la récupération de la description des items de bureau")
            debug(error.message)
            result = ""
        return_result(result)

    def init_thumbs(self):
        """
            appel l'initialisation des onglets utilisateurs dans la bdd
        """
        try:
            initializer = InitThumbs(self.user_datas, self.username, self.server)
            initializer.insert_tabs_and_modules()
        except Exception, error:
            debug("- ERREUR : au contact du serveur de profil  à l'initialisation des onglets utilisateurs")
            debug(error.message)

    def export_thumb(self):
        """
            exporte un onglet vers le gestionnaire de profil
        """
        try:
            exporter = ExportThumbs(self.username, self.session_pt, self.user_id, self.thumb_id, self.thumb_name,
                                self.thumb_indice, self.server, self.get_my_ip())
            return_result(exporter.export_thumb())
        except Exception, error:
            debug("- ERREUR : à l'export d'onglet")
            debug(error.message)
            return_result("")

    def execute_action(self):
        """
            éxécute l'action commandée
        """
        debug("Execute action")
        request_keys = {'desktop':self.get_desktop_items,
                        'thumbs':self.init_thumbs,
                        'exporter':self.export_thumb}
        if self.mode == "desktop": self.get_desktop_items()
        elif self.mode == 'thumbs': self.init_thumbs()
        elif self.mode == "exporter": self.export_thumb()
        else: debug("- ERREUR : unknown procedure Called : %s "% self.mode)

    def get_my_ip(self):
        return LOCAL_IP

def filter_sysargs(sysargs):
    """
        filtre les valeurs passées par ligne de commande
    """
    debug("Les arguments : %s" % str(sysargs))

if __name__ == '__main__':
    try:
        debug("#### Appel du client")
        filter_sysargs(sys.argv)
        client = EnvoleProfilClient(sys.argv)
        client.execute_action()
    except Exception, msg:
        debug("- ERREUR : dans le main.")
        debug(msg.message)
